import 'rxjs/add/operator/map';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { HostConfiguration } from './../../app.constants';
import { BaseHttpService } from '../../utilities/cite-http-service-module/base-http.service';
import { Observable } from 'rxjs/Observable';
import { ExternalSourcesItemModel } from '../../models/external-sources/ExternalSourcesItemModel';
import { BaseCriteria } from '../../models/criteria/BaseCriteria';
import { RequestItem } from '../../models/criteria/RequestItem';
import { ResearcherCriteria } from '../../models/criteria/researchers/ResearcherCriteria';
import { ExternalDatasetCriteria } from '../../models/criteria/external-dataset/ExternalDatasetCriteria';
import { RegistryCriteria } from '../../models/criteria/registry/RegistryCriteria';
import { DataRepositoryCriteria } from '../../models/criteria/data-repository/DataRepositoryCriteria';
import { ServicesCriteria } from '../../models/criteria/services/ServicesCriteria';
import { TagsCriteria } from '../../models/criteria/tags/TagsCriteria';


@Injectable()
export class ExternalSourcesService {

  private actionUrl: string;
  private headers: HttpHeaders;

  constructor(private http: BaseHttpService) {

    this.actionUrl = HostConfiguration.Server + 'external/';

    this.headers = new HttpHeaders();
    this.headers = this.headers.set('Content-Type', 'application/json');
    this.headers = this.headers.set('Accept', 'application/json');
  }

  public searchDatasetRegistry(requestItem: RequestItem<RegistryCriteria>): Observable<ExternalSourcesItemModel[]> {
    return this.http.get<ExternalSourcesItemModel[]>(this.actionUrl + "registries" + "?query=" + requestItem.criteria.like + "&type=" + requestItem.criteria.type, { headers: this.headers });
  }

  public searchDatasetRepository(requestItem: RequestItem<DataRepositoryCriteria>): Observable<ExternalSourcesItemModel[]> {
    return this.http.get<ExternalSourcesItemModel[]>(this.actionUrl + "datarepos" + "?query=" + requestItem.criteria.like + "&type=" + requestItem.criteria.type, { headers: this.headers });
  }

  public searchDatasetService(requestItem: RequestItem<ServicesCriteria>): Observable<ExternalSourcesItemModel[]> {
    return this.http.get<ExternalSourcesItemModel[]>(this.actionUrl + "services" + "?query=" + requestItem.criteria.like + "&type=" + requestItem.criteria.type, { headers: this.headers });
  }

  public searchDatasetTags(requestItem: RequestItem<TagsCriteria>): Observable<ExternalSourcesItemModel[]> {
    return Observable.of([
      { id: "1", name: "Tag 1", description: "" },
      { id: "2", name: "Tag 2", description: "" },
      { id: "3", name: "Tag 3", description: "" },
      { id: "4", name: "Tag 4", description: "" },
      { id: "5", name: "Tag 5", description: "" },
      { id: "6", name: "Tag 6", description: "" },
      { id: "7", name: "Tag 7", description: "" },
      { id: "8", name: "Tag 8", description: "" },
    ])
    //return this.http.get<ExternalSourcesItemModel[]>(this.actionUrl + "tags" + "?query=" + requestItem.criteria.like + "&type=" + requestItem.criteria.type, { headers: this.headers });
  }

  public searchDatasetSExternalDatasetservice(requestItem: RequestItem<ExternalDatasetCriteria>): Observable<ExternalSourcesItemModel[]> {
    return this.http.get<ExternalSourcesItemModel[]>(this.actionUrl + "datasets" + "?query=" + requestItem.criteria.like + "&type=" + requestItem.criteria.type, { headers: this.headers });
  }

  public searchDMPResearchers(requestItem: RequestItem<ResearcherCriteria>): Observable<ExternalSourcesItemModel[]> {
    return this.http.post<ExternalSourcesItemModel[]>(HostConfiguration.Server + "/researchers/getWithExternal", requestItem, { headers: this.headers });
  }

  public searchDMPOrganizations(like: string): Observable<ExternalSourcesItemModel[]> {
    return this.http.get<ExternalSourcesItemModel[]>(this.actionUrl + "organisations" + "?query=" + like, { headers: this.headers });
  }//organizationscriteria.criteria.like

  public searchDMPProfiles(like: string): Observable<ExternalSourcesItemModel[]> {
    return this.http.get<ExternalSourcesItemModel[]>(this.actionUrl + "datasetprofiles/get" + "?query=" + like, { headers: this.headers });
  }

}
